<?php

/**
 *
 *
 * @package    game_showcase
 * @subpackage game_showcase/admin
 * @author     Erik Saulnier <info@eriksaulnier.com>
 */
class Game_Showcase_Admin_Metaboxes {
  /**
   * The ID of this plugin.
   *
   * @since 		1.0.0
   * @access 		private
   * @var 		string 			$plugin_name 		The ID of this plugin.
   */
  private $plugin_name;

  /**
   * The version of this plugin.
   *
   * @since 		1.0.0
   * @access 		private
   * @var 		string 			$version 			The current version of this plugin.
   */
  private $version;

  private $screens = array(
    'game',
  );

  private $fields = array(
    array(
      'label' => 'Author Name(s)',
      'id' => 'authors',
      'default' => 'John Doe',
      'type' => 'text',
    ),
    array(
      'label' => 'Play Link',
      'id' => 'link',
      'type' => 'url',
    ),
    array(
      'label' => 'External Site',
      'id' => 'website',
      'type' => 'url',
    ),
    array(
      'label' => 'Excerpt',
      'id' => 'description',
      'type' => 'textarea',
    ),
    array(
      'label' => 'Image Gallery',
      'id' => 'gallery',
      'type' => 'gallery',
    ),
    array(
      'label' => 'Still in Development',
      'id' => 'flag_dev',
      'type' => 'checkbox',
    ),
  );

  /**
   * Initialize the class and set its properties.
   *
   * @since 		1.0.0
   * @param 		string 			$plugin_name 		The name of this plugin.
   * @param 		string 			$version 			The version of this plugin.
   */
  public function __construct( $plugin_name, $version ) {
    $this->plugin_name = $plugin_name;
    $this->version = $version;
  }

  /**
   * Hooks into WordPress' add_meta_boxes function.
   * Goes through screens (post types) and adds the meta box.
   */
  public function add_meta_boxes() {
    foreach ( $this->screens as $single_screen ) {
      add_meta_box(
        'game_information',
        __( 'Game Information', $this->plugin_name ),
        array( $this, 'add_meta_box_callback' ),
        $single_screen,
        'advanced',
        'high'
      );
    }
  }

  /**
   * Generates the HTML for the meta box
   *
   * @param object $post WordPress post object
   */
  public function add_meta_box_callback( $post ) {
    wp_nonce_field( 'game_information_data', 'game_information_nonce' );
    $this->generate_fields( $post );
  }

  /**
   * Generates the field's HTML for the meta box.
   */
  public function generate_fields( $post ) {
    $output = '';
    foreach ( $this->fields as $field ) {
      $label = '<label for="' . $field['id'] . '">' . $field['label'] . '</label>';
      $db_value = get_post_meta( $post->ID, 'game_information_' . $field['id'], true );
      switch ( $field['type'] ) {
        case 'textarea':
          $input = sprintf(
            '<textarea class="large-text" id="%s" name="%s" rows="5">%s</textarea>',
            $field['id'],
            $field['id'],
            $db_value
          );
          break;
        case 'checkbox':
          $input = sprintf(
            '<input id="%s" name="%s" type="checkbox" %s/>',
            $field['id'],
            $field['id'],
            $db_value == 'on' ? 'checked' : ''
          );
          break;
        case 'gallery':
          // If there are images in the db, load them
          $meta_content = null;
          if ( $db_value ) {
            $meta_content = '<ul class="game_showcase_admin_gallery_list">';
            $value_array = explode(',', $db_value);
            foreach ($value_array as $gallery_item) {
              $meta_content .= '<li>
                <div class="game_showcase_admin_gallery_container">
                  <span class="game_showcase_admin_gallery_close" title="Click to remove">
                    <img id="' . esc_attr($gallery_item) . '" src="' . wp_get_attachment_thumb_url($gallery_item) . '">
                  </span>
                </div>
              </li>';
            }
            $meta_content .= '</ul>';
          }

          // Build the input form
          $input = sprintf(
            '<input id="%s" name="%s" type="%s" value="%s">',
            $field['id'],
            $field['id'],
            'hidden',
            $db_value
          );
          $input .= '<span id="game_showcase_admin_gallery_src">' . $meta_content . '</span>';
          $input .= '<div class="game_showcase_admin_gallery_button_container">
            <input id="game_showcase_admin_gallery_button" class="button" type="button" value="Select Images" />
          </div>';
          break;
        default:
          $input = sprintf(
            '<input %s id="%s" name="%s" type="%s" value="%s">',
            $field['type'] !== 'color' ? 'class="regular-text"' : '',
            $field['id'],
            $field['id'],
            $field['type'],
            $db_value
          );
      }
      $output .= $this->row_format( $label, $input );
    }
    echo '<table class="form-table"><tbody>' . $output . '</tbody></table>';
  }

  /**
   * Generates the HTML for table rows.
   */
  public function row_format( $label, $input ) {
    return sprintf(
      '<tr><th scope="row">%s</th><td>%s</td></tr>',
      $label,
      $input
    );
  }
  /**
   * Hooks into WordPress' save_post function
   */
  public function save_post( $post_id ) {
    if ( ! isset( $_POST['game_information_nonce'] ) )
      return $post_id;

    $nonce = $_POST['game_information_nonce'];
    if ( !wp_verify_nonce( $nonce, 'game_information_data' ) )
      return $post_id;

		if ( $showcase_id = $_POST['game_showcase'] ) {
			$term = get_term_by( 'id', $showcase_id, 'game_showcase' );

			if ( ! empty( $term ) && ! is_wp_error( $term ) ) {
				wp_set_object_terms( $post_id, $term->term_id, 'game_showcase', false );
			}
		}

    if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE )
      return $post_id;

    foreach ( $this->fields as $field ) {
      if ( isset( $_POST[ $field['id'] ] ) ) {
        switch ( $field['type'] ) {
          case 'email':
            $_POST[ $field['id'] ] = sanitize_email( $_POST[ $field['id'] ] );
            break;
          case 'text':
            $_POST[ $field['id'] ] = sanitize_text_field( $_POST[ $field['id'] ] );
            break;
        }

        update_post_meta( $post_id, 'game_information_' . $field['id'], $_POST[ $field['id'] ] );
      } else if ( $field['type'] === 'checkbox' ) {
        update_post_meta( $post_id, 'game_information_' . $field['id'], '0' );
      }
    }
  }

}
