<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://eriksaulnier.com
 * @since      1.0.0
 *
 * @package    Game_Showcase
 * @subpackage Game_Showcase/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Game_Showcase
 * @subpackage Game_Showcase/admin
 * @author     Erik Saulnier <info@eriksaulnier.com>
 */
class Game_Showcase_Admin {

  /**
   * The ID of this plugin.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $plugin_name    The ID of this plugin.
   */
  private $plugin_name;

  /**
   * The version of this plugin.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $version    The current version of this plugin.
   */
  private $version;

  /**
   * Initialize the class and set its properties.
   *
   * @since    1.0.0
   * @param      string    $plugin_name       The name of this plugin.
   * @param      string    $version    The version of this plugin.
   */
  public function __construct( $plugin_name, $version ) {
    $this->plugin_name = $plugin_name;
    $this->version = $version;
  }

  /**
   * Register the stylesheets for the admin side of the site.
   *
   * @since    1.0.0
   */
  public function enqueue_styles() {
    wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/game-showcase-admin.css', array( ), $this->version, 'all' );
  }

  /**
   * Register the JavaScript for the admin side of the site.
   *
   * @since    1.0.0
   */
  public function enqueue_scripts() {
    wp_enqueue_media();
    wp_enqueue_script( 'media-upload' );
    wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/game-showcase-admin.js', array( 'jquery', 'media-upload' ), $this->version, false );
  }

  /**
   * Registers custom post types for the plugin.
   *
   * @since   1.0.0
   */
  public function register_post_types() {
    $labels = array(
      'name' => __( 'Games', 'Post Type General Name', 'game-showcase' ),
      'singular_name' => __( 'Game', 'Post Type Singular Name', 'game-showcase' ),
      'menu_name' => __( 'Games', 'game-showcase' ),
      'name_admin_bar' => __( 'Game', 'game-showcase' ),
      'archives' => __( 'Game Archives', 'game-showcase' ),
      'attributes' => __( 'Game Attributes', 'game-showcase' ),
      'parent_item_colon' => __( 'Parent Game:', 'game-showcase' ),
      'all_items' => __( 'All Games', 'game-showcase' ),
      'add_new_item' => __( 'Add New Game', 'game-showcase' ),
      'add_new' => __( 'Add New', 'game-showcase' ),
      'new_item' => __( 'New Game', 'game-showcase' ),
      'edit_item' => __( 'Edit Game', 'game-showcase' ),
      'update_item' => __( 'Update Game', 'game-showcase' ),
      'view_item' => __( 'View Game', 'game-showcase' ),
      'view_items' => __( 'View Games', 'game-showcase' ),
      'search_items' => __( 'Search Game', 'game-showcase' ),
      'not_found' => __( 'Not found', 'game-showcase' ),
      'not_found_in_trash' => __( 'Not found in Trash', 'game-showcase' ),
      'featured_image' => __( 'Thumbnail', 'game-showcase' ),
      'set_featured_image' => __( 'Set thumbnail', 'game-showcase' ),
      'remove_featured_image' => __( 'Remove thumbnail', 'game-showcase' ),
      'use_featured_image' => __( 'Use as thumbnail', 'game-showcase' ),
      'insert_into_item' => __( 'Insert into Game', 'game-showcase' ),
      'uploaded_to_this_item' => __( 'Uploaded to this Game', 'game-showcase' ),
      'items_list' => __( 'Games list', 'game-showcase' ),
      'items_list_navigation' => __( 'Games list navigation', 'game-showcase' ),
      'filter_items_list' => __( 'Filter Games list', 'game-showcase' ),
    );
    $args = array(
      'label' => __( 'Game', 'game-showcase' ),
      'description' => __( '', 'game-showcase' ),
      'labels' => $labels,
      'menu_icon' => 'dashicons-screenoptions',
      'supports' => array('title', 'editor', 'thumbnail', 'revisions', 'comments' ),
      'taxonomies' => array('game_showcase', 'game_genre' ),
      'public' => true,
      'show_ui' => true,
      'show_in_menu' => true,
      'menu_position' => 5,
      'show_in_admin_bar' => true,
      'show_in_nav_menus' => true,
      'can_export' => true,
      'has_archive' => true,
      'hierarchical' => false,
      'exclude_from_search' => false,
      'show_in_rest' => true,
      'publicly_queryable' => true,
      'capability_type' => 'post',
    );
    register_post_type( 'game', $args );
  }

  /**
   * Registers taxonomies for the plugin.
   *
   * @since   1.0.0
   */
  public function register_taxonomies() {
    // Showcase Taxonomy
    $labels = array(
      'name'              => _x( 'Showcase', 'taxonomy general name', 'game-showcase' ),
      'singular_name'     => _x( 'Showcase', 'taxonomy singular name', 'game-showcase' ),
      'search_items'      => __( 'Search Showcases', 'game-showcase' ),
      'all_items'         => __( 'All Showcases', 'game-showcase' ),
      'parent_item'       => __( 'Parent Showcase', 'game-showcase' ),
      'parent_item_colon' => __( 'Parent Showcase:', 'game-showcase' ),
      'edit_item'         => __( 'Edit Showcase', 'game-showcase' ),
      'update_item'       => __( 'Update Showcase', 'game-showcase' ),
      'add_new_item'      => __( 'Add New Showcase', 'game-showcase' ),
      'new_item_name'     => __( 'New Showcase Name', 'game-showcase' ),
      'menu_name'         => __( 'Showcases', 'game-showcase' ),
    );
    $args = array(
      'labels' => $labels,
      'description' => __( '', 'game-showcase' ),
      'hierarchical' => false,
      'public' => true,
      'publicly_queryable' => true,
      'show_ui' => true,
      'show_in_menu' => true,
      'show_in_nav_menus' => true,
      'show_in_rest' => false,
      'show_tagcloud' => false,
      'show_in_quick_edit' => true,
      'show_admin_column' => true,
      'meta_box_cb' => array( $this, 'showcase_dropdown' ),
    );
    register_taxonomy( 'game_showcase', array( 'game' ), $args );

    // Genre Taxonomy
    $labels = array(
      'name'              => _x( 'Genres', 'taxonomy general name', 'game-showcase' ),
      'singular_name'     => _x( 'Genre', 'taxonomy singular name', 'game-showcase' ),
      'search_items'      => __( 'Search Genres', 'game-showcase' ),
      'all_items'         => __( 'All Genres', 'game-showcase' ),
      'parent_item'       => __( 'Parent Genre', 'game-showcase' ),
      'parent_item_colon' => __( 'Parent Genre:', 'game-showcase' ),
      'edit_item'         => __( 'Edit Genre', 'game-showcase' ),
      'update_item'       => __( 'Update Genre', 'game-showcase' ),
      'add_new_item'      => __( 'Add New Genre', 'game-showcase' ),
      'new_item_name'     => __( 'New Genre Name', 'game-showcase' ),
      'menu_name'         => __( 'Genres', 'game-showcase' ),
    );
    $args = array(
      'labels' => $labels,
      'description' => __( '', 'game-showcase' ),
      'hierarchical' => true,
      'public' => true,
      'publicly_queryable' => true,
      'show_ui' => true,
      'show_in_menu' => true,
      'show_in_nav_menus' => true,
      'show_in_rest' => false,
      'show_tagcloud' => false,
      'show_in_quick_edit' => true,
      'show_admin_column' => true,
    );
    register_taxonomy( 'game_genre', array( 'game' ), $args );
  }

	/**
   * Construct showcase dropdown selection for games
   *
   * @since   1.0.0
   */
  public function showcase_dropdown( $post, $box ) {
    $defaults = array('taxonomy' => 'game_showcase');
    if ( !isset($box['args']) || !is_array($box['args']) )
      $args = array();
    else
      $args = $box['args'];

    extract( wp_parse_args($args, $defaults), EXTR_SKIP );

    $tax = get_taxonomy($taxonomy);
    $name = ( $taxonomy == 'game_showcase' ) ? 'game_showcase' : 'tax_input[' . $taxonomy . ']';
    $term_obj = wp_get_object_terms( $post->ID, $taxonomy );
    if ( !empty( $term_obj ) )
      $selected = $term_obj[0]->term_id;
    else
      $selected = '';
    ?>
      <div id="taxonomy-<?php echo $taxonomy; ?>" class="acf-taxonomy-field categorydiv">

        <input type='hidden' name='<?php echo $name; ?>[]' value='0' />

        <?php wp_dropdown_categories( array(
          'taxonomy' => $taxonomy,
          'hide_empty' => 0,
          'name' => $name,
          'selected' => $selected,
          'orderby' => 'name',
          'hierarchical' => 0,
          'show_option_none' => '&mdash;',
          'class' => 'admin-input-fullwidth',
        ) ); ?>

      </div>
    <?php
  }

  /**
   * Modify the columns displayed on the game edit page.
   *
   * @since    1.0.0
   */
  public function add_custom_post_type_columns( $columns ) {
    $columns = array(
      'cb' => '<input type="checkbox" />',
      'title' => __( 'Game Title' ),
      'authors' => __( 'Authors' ),
      'showcase' => __( 'Showcase' ),
      'genre' => __( 'Genres' ),
      'date' => __( 'Date' )
    );

    return $columns;
  }

  /**
   * Make select columns sortable.
   *
   * @since    1.0.0
   */
  public function custom_post_type_sortable_columns( $columns ) {
    $columns['authors'] = 'authors';
    $columns['showcase'] = 'game_showcase';
    $columns['genre'] = 'game_genre';

    return $columns;
  }

  /**
   * The display function for our custom columns.
   *
   * @since    1.0.0
   */
  public function custom_post_type_columns( $column ) {
    global $post;

    switch ($column) {
      case 'authors':
        $authors = get_post_meta( $post->ID, 'game_information_authors', true );

        if ( !empty( $authors ) ) {
          _e( $authors );
        } else {
          _e( 'No Authors Listed' );
        }
        break;
      case 'showcase':
        $terms = get_the_terms( $post->ID, 'game_showcase' );

        if ( !empty( $terms ) ) {
          $out = array();

          foreach ( $terms as $term ) {
            $out[] = sprintf( '<a href="%s">%s</a>',
              esc_url( add_query_arg( array(
                'taxonomy' => 'game_showcase',
                'post_type' => $post->post_type,
                'tag_ID' => $term->term_id
              ), 'term.php' ) ),
              esc_html( sanitize_term_field( 'name', $term->name, $term->term_id, 'game_showcase', 'display' ) )
            );
          }

          echo join( ', ', $out );
        } else {
          _e( 'No Showcases' );
        }
        break;
      case 'genre':
        $terms = get_the_terms( $post->ID, 'game_genre' );

        if ( !empty( $terms ) ) {
          $out = array();

          foreach ( $terms as $term ) {
            $out[] = sprintf( '<a href="%s">%s</a>',
              esc_url( add_query_arg( array(
                'taxonomy' => 'game_genre',
                'post_type' => $post->post_type,
                'tag_ID' => $term->term_id
              ), 'term.php' ) ),
              esc_html( sanitize_term_field( 'name', $term->name, $term->term_id, 'game_genre', 'display' ) )
            );
          }

          echo join( ', ', $out );
        } else {
          _e( 'No Genres' );
        }
        break;
      default:
        break;
    }
  }

  /**
   * This is where we call our sort columns function.
   *
   * @since    1.0.0
   */
  function custom_edit_screen_sort() {
    add_filter( 'request', array( $this, 'sort_custom_post_type' ) );
  }

  /**
   * The functions for sorting our custom columns.
   *
   * @since    1.0.0
   */
  function sort_custom_post_type( $vars ) {
    if ( isset( $vars['post_type'] ) && 'game' == $vars['post_type'] && isset( $vars['orderby'] ) ) {

      if ( 'authors' == $vars['orderby'] ) {
        $vars = array_merge(
          $vars,
          array(
            'meta_key' => 'game_information_authors',
            'orderby' => 'meta_value'
          )
        );
      }

    }

    return $vars;
  }

  /**
   * Puts the plugin's advanced meta fields right after the title on the edit screen
   *
   * @since   1.0.0
   */
  public function put_advanced_meta_after_title() {
    global $post, $wp_meta_boxes;
    do_meta_boxes(get_current_screen(), 'advanced', $post);
    unset($wp_meta_boxes['game']['advanced']);
  }

}
