<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://eriksaulnier.com
 * @since      1.0.0
 *
 * @package    Game_Showcase
 * @subpackage Game_Showcase/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Game_Showcase
 * @subpackage Game_Showcase/public
 * @author     Erik Saulnier <info@eriksaulnier.com>
 */
class Game_Showcase_Public {

  /**
   * The ID of this plugin.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $plugin_name    The ID of this plugin.
   */
  private $plugin_name;

  /**
   * The version of this plugin.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $version    The current version of this plugin.
   */
  private $version;

  private $form_submitted = false;
  private $url_params = "/";

  /**
   * Initialize the class and set its properties.
   *
   * @since    1.0.0
   * @param      string    $plugin_name       The name of the plugin.
   * @param      string    $version    The version of this plugin.
   */
  public function __construct( $plugin_name, $version ) {
    $this->plugin_name = $plugin_name;
    $this->version = $version;
  }

  /**
   * Register the stylesheets for the public-facing side of the site.
   *
   * @since    1.0.0
   */
  public function enqueue_styles() {
    wp_enqueue_style( 'featherlight', plugin_dir_url( __FILE__ ) . 'css/featherlight.min.css', array() );
    wp_enqueue_style( 'featherlight-gallery', plugin_dir_url( __FILE__ ) . 'css/featherlight.gallery.min.css', array() );

    wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/game-showcase-public.css', array(), $this->version, 'all' );
  }

  /**
   * Register the JavaScript for the public-facing side of the site.
   *
   * @since    1.0.0
   */
  public function enqueue_scripts() {
    wp_enqueue_script( 'featherlight', plugin_dir_url( __FILE__ ) . 'js/featherlight.min.js', array( 'jquery' ) );
    wp_enqueue_script( 'featherlight-gallery', plugin_dir_url( __FILE__ ) . 'js/featherlight.gallery.min.js', array( 'featherlight' ) );

    wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/game-showcase-public.js', array( 'jquery', 'featherlight-gallery' ), $this->version, false );
  }

  /**
   * Overrides the page template for games.
   *
   * @since    1.0.0
   */
  public function custom_post_type_single_template( $template ) {
    global $post;

    $template_name = '/single-game.php';

    // Check for tempalte in theme first, and load ours if not found
    if ( file_exists( get_template_directory() . $template_name ) ) {
      $template = get_template_directory() . $template_name;
    } else if ( $post->post_type == 'game' ) {
      $template = dirname( __FILE__ ) . '/partials' . $template_name;
    }

    return $template;
  }

  public function check_header_for_filters() {
    if ( isset($_POST['submitted']) ) {
      if ( $_POST['submitted'] == '1' ) {
        $this->form_submitted = true;
        $this->url_params .= '?post_type=game';

        if ( isset($_POST['search']) && !empty($_POST['search']) ) {
          $search = trim(stripslashes($_POST['search']));
          $search_terms = explode(' ', $search);
          $search_query = implode('+', $search_terms);

          // If there is a search query add it to the url params
          if ($search_query) {
            $this->url_params .= "&s=$search_query";
          }
        }

        // If showcases specified
        if ( isset($_POST['game_showcase']) && !empty($_POST['game_showcase']) ) {
          $showcases = $_POST['game_showcase'];
          $showcases_query = implode(',', $showcases); // ',' means OR

          // If there is a showcase query add it to the url params
          if ($showcases_query) {
            $this->url_params .= "&game_showcase=$showcases_query";
          }
        }

        // If genres specified
        if ( isset($_POST['game_genre']) && !empty($_POST['game_genre']) ) {
          $genres = $_POST['game_genre'];
          $genres_query = implode('+', $genres); // '+' means AND

          // If there is a genre query add it to the url params
          if ($genres_query) {
            $this->url_params .= "&game_genre=$genres_query";
          }
        }

        // If empty query
        if ($this->url_params == '/') {
          $this->url_params = '/?s=';
        }

        wp_redirect(home_url().$this->url_params);
      }
    }
  }

  public function custom_post_type_search( $query ) {
    if ( ( is_search() || is_post_type_archive() ) && $query->is_main_query() && $query->get( 's' ) ) {
      $query->set( 'orderby', 'date' );
      $query->set( 'order', 'ASC' );

      if ( $query->get( 'post_type' ) == 'game' ) {
        $custom_fields = array(
          'game_information_authors',
          'game_information_description'
        );

        $search_term = $query->query_vars['s'];
        $query->set( 's', '' );
        $query->set( 'q', $search_term );

        if ( $search_term != '' ) {
          $meta_query = array( 'relation' => 'OR' );
          foreach ($custom_fields as $cf) {
            array_push($meta_query, array(
              'key' => $cf,
              'value' => $search_term,
              'compare' => 'LIKE'
            ));
          }

          $query->set( 'meta_query', $meta_query );
          $query->set( 'post_type', 'game' );
        }
      }
    }
  }

  public function custom_posts_where( $where, $query ) {
    global $wpdb;

    if ( $query->query_vars['post_type'] && $query->query_vars['post_type'][0] == 'game' ) {
      $where .= ' OR ' . $wpdb->posts . '.post_title LIKE \'%' . esc_sql( like_escape( $_GET['s'] ) ) . '%\'';
    }

    return $where;
  }

}
